function u=SplitBregDebP_PS(g,ker,blur,mu,lambda,frame,Level,wLevel,gamma,Gamma,maxit,oimg,shrinktype)

[D,R]=GenerateFrameletFilter(frame);
W  = @(x) FraDecMultiLevel(x,D,Level); % Frame decomposition
WT = @(x) FraRecMultiLevel(x,R,Level); % Frame reconstruction
[m,n]=size(g);
cker=rot90(ker,2);

BTg=blur(g,cker);

F=@(x)fft2(x);
FT=@(x)ifft2(x); 

%Compute the weighted thresholding parameters.
muLevel1=getwThresh(gamma/mu,wLevel,Level,D);
muLevel2=getwThresh(mu/(mu+2*lambda),wLevel,Level,D);

%Compute the eigenvalues of P.
eigenP=eigenofP(ker,mu,m,n);  

normg=norm(g,'fro');

%initialization
u=zeros(m,n);
b=W(u);
d=b;

nD=length(D);

for nstep=1:maxit 

    for innerstep=1:1
        % solve u.
        C=CoeffOper('-',d,b);
        u=FT(F(mu*WT(C)+BTg)./eigenP);
        u(u>255)=255;u(u<0)=0;
    
        % solve d
        C=W(u);
        Cpb=CoeffOper('+',C,b);
        
        % split Cpb into l1 and l2 two parts d1 and d2
        for ki=1:Level
            for ji=1:nD-1
                for jj=1:nD-1
                    Cpb1{ki}{ji,jj}=zeros(size(Gamma));Cpb2{ki}{ji,jj}=zeros(size(Gamma));
                    Cpb1{ki}{ji,jj}(logical(Gamma))=Cpb{ki}{ji,jj}(logical(Gamma));
                    Cpb2{ki}{ji,jj}(logical(1-Gamma))=Cpb{ki}{ji,jj}(logical(1-Gamma));
                end
            end
        end        
        d1=CoeffOper(shrinktype,Cpb1,muLevel1);
        d2=CoeffOper('hm',Cpb2,muLevel2);
        d=CoeffOper('cb',d1,d2,Gamma);
    end
%     imshow(u,[]);title(['Iter = ' num2str(nstep)]);drawnow;

    error=0;
    for ki=1:Level
        for ji=1:nD-1
            for jj=1:nD-1
                if ((ji~=1)||(jj~=1))||(ki==Level)
                    deltab=C{ki}{ji,jj}-d{ki}{ji,jj};
                    error=error+norm(deltab,'fro')^2;
                    b{ki}{ji,jj}=b{ki}{ji,jj}+deltab;
                end
            end
        end
    end

    error=sqrt(error)/normg;
    if error<1e-4
        disp(['Subproblem Step = ' num2str(nstep) '; PSNR = ' num2str(psnr(u/255,oimg/255)) '; error = ' num2str(error)]);
        break;
    end
end
if error>=1e-4
    disp(['Subproblem Step = ' num2str(nstep) '; PSNR = ' num2str(psnr(u/255,oimg/255)) '; error = ' num2str(error)]);
end

function eigenP=eigenofP(ker,mu,m,n)

[nker,mker]=size(ker);
tmp=zeros(m,n);tmp(1:nker,1:mker)=ker;
tmp=circshift(tmp,[-floor(nker/2),-floor(mker/2)]);
eigenP=abs(fft2(tmp)).^2+mu;

function [D,R]=GenerateFrameletFilter(frame)
if frame==0          %Haar Wavelet
    D{1}=[0 1 1]/2;
    D{2}=[0 1 -1]/2;
    D{3}='cc';
    R{1}=[1 1 0]/2;
    R{2}=[-1 1 0]/2;
    R{3}='cc';
elseif frame==1      %Piecewise Linear Framelet
    D{1}=[1 2 1]/4;
    D{2}=[1 0 -1]/4*sqrt(2);
    D{3}=[-1 2 -1]/4;
    D{4}='ccc';
    R{1}=[1 2 1]/4;
    R{2}=[-1 0 1]/4*sqrt(2);
    R{3}=[-1 2 -1]/4;
    R{4}='ccc';
elseif frame==3      %Piecewise Cubic Framelet
    D{1}=[1 4 6 4 1]/16;
    D{2}=[1 2 0 -2 -1]/8;
    D{3}=[-1 0 2 0 -1]/16*sqrt(6);
    D{4}=[-1 2 0 -2 1]/8;
    D{5}=[1 -4 6 -4 1]/16;
    D{6}='ccccc';
    R{1}=[1 4 6 4 1]/16;
    R{2}=[-1 -2 0 2 1]/8;
    R{3}=[-1 0 2 0 -1]/16*sqrt(6);
    R{4}=[1 -2 0 2 -1]/8;
    R{5}=[1 -4 6 -4 1]/16;
    R{6}='ccccc';
end